## 
# Drought Event
# Author - David Fidge
#
# Version 1.4
# - added new landmark code
# Version 1.3
# Initial release
#

from CvPythonExtensions import *
from RandomEventsConstants import *

import CvScreenEnums
import GlobalFunctions
import LandmarkManager
import RandomEventsIO
import CvUtil
import random

#from CvRandomEventsScreenEnums import *
#import CvRandomEventsScreenUtils
#import RandomEventsScreen
#import math

GlobalFunctions = GlobalFunctions.GlobalFunctions()
LandmarkManager = LandmarkManager.LandmarkManager()
gc = CyGlobalContext()
CyMapInstance = gc.getMap()


#the main class

class RandomEventDrought:
	"Drought Event Class"
	
	#class initialisation (called on creation of the class instance within CvScreensInterface)
	
	def __init__(self, io):

		self.desertList = []
		self.droughtPlot = None
		self.droughtLength = -1

		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started

		self.IOinstance = io

	#initialise the mod.  This is called when loading a game or starting a new game
	def initialise(self):
		"initialise the mod.  This is called when loading a game or starting a new game"

		#get saved drought variables
		px, py, self.droughtLength = self.IOinstance.getDroughtTuple()
		self.droughtPlot = CyMapInstance.plot(px,py)

		#since the landmark and bonuses are saved automatically, dont need to do anything else.  


	def isDroughtInProgress(self):
		"returns true if drought is in progress, false otherwise"
		if (self.droughtLength >= 0):
			return true
		
		return false


	def startDrought(self):
		"set up a new drought"

		self.buildDesertList()

		#if no desert squares then return false
		if (len(self.desertList) == 0):
			return false

		#get a random desert plot that will act as the centre of the drought
		self.droughtPlot = self.desertList[random.randint(0, len(self.desertList) - 1)]

		self.droughtLength = random.randint(DROUGHT_MAXLENGTH / 2, DROUGHT_MAXLENGTH)

		DROUGHT_RADIUS = random.randint(DROUGHT_MINRADIUS, DROUGHT_MAXRADIUS)

		#set a landmark at centre of drought
		LandmarkManager.addLandmarkText(self.droughtPlot, "Drought, Radius %d" %(DROUGHT_RADIUS))

		#inflict the drought onto squares within radius (any squares with existing bonuses are ignored)
		for x in range (-DROUGHT_RADIUS, DROUGHT_RADIUS+1):
			for y in range (-DROUGHT_RADIUS, DROUGHT_RADIUS+1):
				tempx = GlobalFunctions.getNextX(self.droughtPlot.getX(), x)
				tempy = GlobalFunctions.getNextY(self.droughtPlot.getY(), y)

				if (tempx != -1 and tempy != -1):

					newplot = (CyMapInstance.plot(tempx, tempy))

					if (newplot.getBonusType(CyGame().getActivePlayer()) == gc.getInfoTypeForString("NO_BONUS") and not newplot.isWater()):
						newplot.setBonusType(gc.getInfoTypeForString("BONUS_DROUGHT"))


		#store drought vars
		self.IOinstance.setDroughtTuple((self.droughtPlot.getX(),self.droughtPlot.getY(),self.droughtLength))


		#show message
		if (DROUGHT_RADIUS <= DROUGHT_MINRADIUS + ((DROUGHT_MAXRADIUS - DROUGHT_MINRADIUS) / 2)):
			droughtMsg = "A drought has struck!  Without water, the crops wither in the searing heat.  Food output -1 on affected squares for the duration"
		else: 
			droughtMsg = "A searing drought has struck!  Widespread loss of crops cause massive food shortages!  Food output -1 on affected squares for the duration"

		#only show message if the main drought plot is visible to the human (landmark will always be there in any case if human reveals later)
		if (self.droughtPlot.isVisibleToWatchingHuman() == true):
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, droughtMsg, "AS2D_DROUGHT", 2, "Art/Interface/Buttons/BaseTerrain/Desert.dds", ColorTypes(7), self.droughtPlot.getX(), self.droughtPlot.getY(), True, True) 
		return true

	
	def processDrought(self):
		"do next turn for drought"

		#update drought vars
		self.droughtLength -= 1
		if (self.droughtLength >= 0):
			self.IOinstance.setDroughtTuple((self.droughtPlot.getX(),self.droughtPlot.getY(),self.droughtLength))
		else:
			self.stopDrought()
		


	def stopDrought(self):
		"stop a drought currently in progress"

		#save vars for no drought
		self.IOinstance.setDroughtTuple((-1,-1,-1))

		LandmarkManager.removeLandmarkText(self.droughtPlot, "Drought")

		#restore droughted squares back to normal 
		#safe to go with maxradius here even if it didnt extend that far
		for x in range (-DROUGHT_MAXRADIUS, DROUGHT_MAXRADIUS+1):
			for y in range (-DROUGHT_MAXRADIUS, DROUGHT_MAXRADIUS+1):
				tempx = GlobalFunctions.getNextX(self.droughtPlot.getX(), x)
				tempy = GlobalFunctions.getNextY(self.droughtPlot.getY(), y)

				if (tempx != -1 and tempy != -1):

					newplot = (CyMapInstance.plot(tempx, tempy))

					if (newplot.getBonusType(CyGame().getActivePlayer()) == gc.getInfoTypeForString("BONUS_DROUGHT") and not newplot.isWater()):
						newplot.setBonusType(gc.getInfoTypeForString("NO_BONUS"))

		#show message
		droughtMsg = "The drought has ended.  The rains have come at last!"

		#only show message if the main drought plot is visible to the human (landmark will always be there in any case if human reveals later)
		if (self.droughtPlot.isVisibleToWatchingHuman() == true):
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, droughtMsg, "", 2, "Art/Interface/Buttons/BaseTerrain/Desert.dds", ColorTypes(7), self.droughtPlot.getX(), self.droughtPlot.getY(), True, True) 
		return true

		
	def buildDesertList(self):
		"builds a list of all deserts on the map"

		#clear the existing desertList
		self.desertList = []

		maxX = CyMapInstance.getGridWidth() 
		maxY = CyMapInstance.getGridHeight()

		#loop through all plots on the map and add to list if it is a desert
		for x in range(maxX):
			for y in range(maxY):
				newplot = CyMapInstance.plot(x, y)
				#for some reason I once got a desert plot on a peak
        			if (newplot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_DESERT") and not newplot.isPeak()):
					#add the desert to the list
					self.desertList.append(newplot)
						

	def turnChecker(self, iTurnNum):
		"this is called every turn.  Checks to see if the drought event should occur this turn"


		if (self.isDroughtInProgress()):
			self.processDrought()
        	else:
        		
        		#CvUtil.pyPrint("entered turnchecker")
        
        		#the chance of a drought is only determined every DROUGHT_DELAY turns
        
        		if (iTurnNum % DROUGHT_DELAY == 0):
        
        			randnum = random.randint(1, DROUGHT_MAX_CHANCE)
        
        			if (randnum <= DROUGHT_CHANCE_LIMIT):
        
        				setupSuccess = false
        
                			setupSuccess = self.startDrought()

					#dont care if the drought setup failed or not
                

