## 
# Fertile Event
# Author - David Fidge
#
# Version 1.4
# - added new landmark code
# Version 1.3
# Initial release
#

from CvPythonExtensions import *
from RandomEventsConstants import *

import CvScreenEnums
import GlobalFunctions
import LandmarkManager
import RandomEventsIO
import CvUtil
import random

#from CvRandomEventsScreenEnums import *
#import CvRandomEventsScreenUtils
#import RandomEventsScreen
#import math

GlobalFunctions = GlobalFunctions.GlobalFunctions()
LandmarkManager = LandmarkManager.LandmarkManager()
gc = CyGlobalContext()
CyMapInstance = gc.getMap()

#the main class

class RandomEventFertile:
	"Fertile Event Class"
	
	#class initialisation (called on creation of the class instance within CvScreensInterface)
	
	def __init__(self, io):

		self.grasslandList = []
		self.fertilePlot = None
		self.fertileLength = -1

		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started

		self.IOinstance = io

	#initialise the mod.  This is called when loading a game or starting a new game
	def initialise(self):
		"initialise the mod.  This is called when loading a game or starting a new game"

		#get saved fertile variables
		px, py, self.fertileLength = self.IOinstance.getFertileTuple()
		self.fertilePlot = CyMapInstance.plot(px,py)

		#since the landmark and bonuses are saved automatically, dont need to do anything else.  


	def isFertileInProgress(self):
		"returns true if fertile is in progress, false otherwise"
		if (self.fertileLength >= 0):
			return true
		
		return false


	def startFertile(self):
		"set up a new fertile"

		self.buildGrasslandList()

		#if no desert squares then return false
		if (len(self.grasslandList) == 0):
			return false

		#get a random desert plot that will act as the centre of the fertile
		self.fertilePlot = self.grasslandList[random.randint(0, len(self.grasslandList) - 1)]

		self.fertileLength = random.randint(FERTILE_MAXLENGTH / 2, FERTILE_MAXLENGTH)

		FERTILE_RADIUS = random.randint(FERTILE_MINRADIUS, FERTILE_MAXRADIUS)

		#set a landmark at centre of fertile
		LandmarkManager.addLandmarkText(self.fertilePlot, "Fertility, Radius %d" %(FERTILE_RADIUS))

		#inflict the fertility onto squares within radius (any squares with existing bonuses are ignored)
		for x in range (-FERTILE_RADIUS, FERTILE_RADIUS+1):
			for y in range (-FERTILE_RADIUS, FERTILE_RADIUS+1):
				tempx = GlobalFunctions.getNextX(self.fertilePlot.getX(), x)
				tempy = GlobalFunctions.getNextY(self.fertilePlot.getY(), y)

				if (tempx != -1 and tempy != -1):

					newplot = (CyMapInstance.plot(tempx, tempy))

					if (newplot.getBonusType(CyGame().getActivePlayer()) == gc.getInfoTypeForString("NO_BONUS") and not newplot.isWater()):
						newplot.setBonusType(gc.getInfoTypeForString("BONUS_FERTILE"))


		#store fertile vars
		self.IOinstance.setFertileTuple((self.fertilePlot.getX(),self.fertilePlot.getY(),self.fertileLength))

		#show message
		if (FERTILE_RADIUS <= FERTILE_MINRADIUS + ((FERTILE_MAXRADIUS - FERTILE_MINRADIUS) / 2)):
			fertileMsg = "Fertility - Rainfalls in the area are ideal for growing crops this season!  Food output +1 on affected squares for the duration"
		else: 
			fertileMsg = "Fertility - Rainfalls across a wide area are ideal for growing crops this season!  Food output +1 on affected squares for the duration"

		#only show message if the main fertile plot is visible to the human (landmark will always be there in any case if human reveals later)
		if (self.fertilePlot.isVisibleToWatchingHuman() == true):
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, fertileMsg, "AS2D_FERTILE", 2, "Art/Interface/Buttons/BaseTerrain/grassland.dds", ColorTypes(7), self.fertilePlot.getX(), self.fertilePlot.getY(), True, True) 
		return true

	
	def processFertile(self):
		"do next turn for fertile"

		#update fertile vars
		self.fertileLength -= 1
		if (self.fertileLength >= 0):
			self.IOinstance.setFertileTuple((self.fertilePlot.getX(),self.fertilePlot.getY(),self.fertileLength))
		else:
			self.stopFertile()
		


	def stopFertile(self):
		"stop a fertile currently in progress"

		#save vars for no fertile
		self.IOinstance.setFertileTuple((-1,-1,-1))

		LandmarkManager.removeLandmarkText(self.fertilePlot, "Fertility")

		#restore fertileed squares back to normal 
		#safe to go with maxradius here even if it didnt extend that far
		for x in range (-FERTILE_MAXRADIUS, FERTILE_MAXRADIUS+1):
			for y in range (-FERTILE_MAXRADIUS, FERTILE_MAXRADIUS+1):
				tempx = GlobalFunctions.getNextX(self.fertilePlot.getX(), x)
				tempy = GlobalFunctions.getNextY(self.fertilePlot.getY(), y)

				if (tempx != -1 and tempy != -1):

					newplot = (CyMapInstance.plot(tempx, tempy))

					if (newplot.getBonusType(CyGame().getActivePlayer()) == gc.getInfoTypeForString("BONUS_FERTILE") and not newplot.isWater()):
						newplot.setBonusType(gc.getInfoTypeForString("NO_BONUS"))

		#show message
		fertileMsg = "The fertility has ended."

		#only show message if the main fertile plot is visible to the human (landmark will always be there in any case if human reveals later)
		if (self.fertilePlot.isVisibleToWatchingHuman() == true):
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, fertileMsg, "", 2, "Art/Interface/Buttons/BaseTerrain/grassland.dds", ColorTypes(7), self.fertilePlot.getX(), self.fertilePlot.getY(), True, True) 
		return true

		
	def buildGrasslandList(self):
		"builds a list of all grassland plots on the map"

		#clear the existing grasslandList
		self.grasslandList = []

		maxX = CyMapInstance.getGridWidth() 
		maxY = CyMapInstance.getGridHeight()

		#loop through all plots on the map and add to list if it is a desert
		for x in range(maxX):
			for y in range(maxY):
				newplot = CyMapInstance.plot(x, y)
        			if (newplot.getTerrainType() == gc.getInfoTypeForString("TERRAIN_GRASS") and not newplot.isPeak()):
					#add the grassland plot to the list
					self.grasslandList.append(newplot)
						

	def turnChecker(self, iTurnNum):
		"this is called every turn.  Checks to see if the fertile event should occur this turn"


		if (self.isFertileInProgress()):
			self.processFertile()
        	else:
        		
        		#CvUtil.pyPrint("entered turnchecker")
        
        		#the chance of a fertile is only determined every FERTILE_DELAY turns
        
        		if (iTurnNum % FERTILE_DELAY == 0):
        
        			randnum = random.randint(1, FERTILE_MAX_CHANCE)
        
        			if (randnum <= FERTILE_CHANCE_LIMIT):
        
        				setupSuccess = false
        
                			setupSuccess = self.startFertile()

					#dont care if the fertile setup failed or not
                

